
tinymce.PluginManager.add('dmbindevent', function(tinyMCEEditor) {


    let scrollTimer = null;

    // 判断元素是否可视方向，上方可视，或下方可视
    function elementIsInWindow($ele) {
        let windowTop = 0, eleTop = 0, eleHeight = 0, canSeeDirection = 'bottom';
        windowTop = $(document).scrollTop();
        eleTop = $ele.offset().top + 60;
        eleHeight = $ele.height();
        if (eleTop - windowTop > 90) {
            canSeeDirection = 'top'
        } else {
            canSeeDirection = 'bottom'
        }
        
        return canSeeDirection;
    }

    // 判断文本焦点是否在可视范围内，若不是，则需像上滚动
    function bookmarkInWindow () {
        let windowTop = 0, newWindowTop = 0, bookmarkTop = 0, titleHeight = 0, bookmarkNode,
            limitWindowScrollHeight = 0, scrollDistance = 0, pHeight = 0, textModalStatus = false;

        textModalStatus = window.dmEditor.textModalStatus;
        limitWindowScrollHeight = window.dmEditor.keyboardOffsetTop;
        if (!limitWindowScrollHeight || !textModalStatus) {
            return;
        }
        
        bookmarkNode =  tinyMCEEditor.selection.getNode();
        pHeight = $(tinyMCEEditor.dom.select('p')).height();
        windowTop = $(document).scrollTop();
        titleHeight = $('.title-wrap').height();
        bookmarkTop = bookmarkNode.offsetTop + bookmarkNode.offsetHeight + titleHeight;

        limitWindowScrollHeight = titleHeight + limitWindowScrollHeight;

        if (bookmarkTop - windowTop > limitWindowScrollHeight && scrollTimer === null) {
            scrollTimer = setTimeout(function () {
                // 输入文本时，若光标不在可视范围内,app会将光标自动滚动到键盘上方，若是输入数字会在键盘上方较多距离
                newWindowTop = $(document).scrollTop();
                const offsetDistance = bookmarkTop - newWindowTop - limitWindowScrollHeight;
                if (offsetDistance < 0) {
                    scrollTimer = null;
                    return;
                }
                let scrollPTimes = 3;
                if (offsetDistance < 20) {
                    scrollPTimes = 1
                } else if (offsetDistance < 45) {
                    scrollPTimes = 2
                }
                scrollDistance = newWindowTop + (pHeight * scrollPTimes);
                $('html, body').animate({'scrollTop': scrollDistance}, 300);
                clearTimeout(scrollTimer);
                scrollTimer = null;
            }, 200);
        }
    }

    // 删除元素块
    function removeNode($node) {
        const $p = $('<p><br></p>');
        $node.replaceWith($p);
        tinyMCEEditor.undoManager.add();
    }

    //图片操作按钮
    function toggleImgEditButton ($img, canSeeDirection) {
        const node = $img[0];

        if (!node) return;

        const isSmallImg = $img.hasClass('dm_insert_img_small');
        const hiddenEnlarge= isSmallImg ? '' : 'hidden-block';
        const hiddenNarrow = isSmallImg ? 'hidden-block' : '';
        const deleteBtnBottomClass = canSeeDirection === 'bottom' ? 'delete-bottom' : '';
        const editBlockBottomClass = canSeeDirection === 'bottom' ? 'app-edit-img-btn-bottom' : '';
        const editButton = '\
            <div class="app-edit-img-btn-wrap app-edit-block ' + editBlockBottomClass + '">\
                <div class="app-edit-img-btn">\
                    <div class="app-edit-img-btn-content">\
                        <span class="enlarge ' + hiddenEnlarge + '">大图</span>\
                        <span class="narrow ' + hiddenNarrow + '">小图</span>\
                        <span class="node">注释</span>\
                        <span class="edit">编辑</span>\
                    </div>\
                </div>\
            </div>\
        ';
        const deleteButton = '<span class="delete ' + deleteBtnBottomClass + '"></span>';
        $img.append($(editButton));
        $img.find('.pos_positive div:first-child').append($(deleteButton));
    }

    // 点击图片
    function onClickImage ($img, $target) {
        $img.find('img').addClass('show-outline');
        const imgId = $img[0].id;
        const isDeleteBtn = $target.hasClass('delete');
        const isEnlargeBtn = $target.hasClass('enlarge');
        const isNarrowtn = $target.hasClass('narrow');
        const isEditBtn = $target.hasClass('edit');
        const isNodeBtn = $target.hasClass('node');
        const isComment = $target.hasClass('dm_comment');
        const $dom = $(tinyMCEEditor.getBody());
        const canSeeDirection = elementIsInWindow($img);
        const changeDirection = ($img.find('.app-edit-img-btn-bottom').length && canSeeDirection !== 'bottom') || (!$img.find('.app-edit-img-btn-bottom').length && canSeeDirection !== 'top')
        if (isDeleteBtn) {
            removeNode($img);
        } else if (isEnlargeBtn) {
            $img.find('.imgsize_btn').text('小图');
            $img.removeClass('dm_insert_img_small');
            $target.addClass('hidden-block');
            $target.next().removeClass('hidden-block');
            tinyMCEEditor.execCommand('mceAutoResize');
        } else if (isNarrowtn) {
            $img.find('.imgsize_btn').text('大图');
            $img.addClass('dm_insert_img_small');
            $target.addClass('hidden-block');
            $target.prev().removeClass('hidden-block');
        } else if (isEditBtn) {
            const imageUrl = $img.find('img').attr('src');
            let params = {
                imgId: imgId,
                imageUrl: imageUrl
            };
            params = JSON.stringify(params);
            // 调用APP方法
            if (window.dmEditor.editorClickEditImage) {
                window.dmEditor.editorClickEditImage(params);
            }
        } else if (isNodeBtn || isComment) {
            const node = $img.find('.dm_comment').text();

            if (isComment && !node) {
                return false;
            }

            let params = {
                imgId: imgId,
                node: node
            };
            params = JSON.stringify(params);
            // 调用APP方法
            if (window.dmEditor.editorClickCommentImage) {
                window.dmEditor.editorClickCommentImage(params);
            }
        } else {
            const hasAppEditImgBtn = $img.find('.app-edit-img-btn-wrap').length > 0;
            if (!hasAppEditImgBtn) {
                toggleImgEditButton($img, canSeeDirection);
            }
            if (changeDirection) {
                $dom.find('.app-edit-block').remove();
                $dom.find('.delete').remove();
                toggleImgEditButton($img, canSeeDirection);
            }
        }
    }
    function removeEditBtn ($target) {
        const $dom = $(tinyMCEEditor.getBody());
        const contentEditBtn = $dom.find('.app-edit-block').length > 0;
        const targetEditBtn = $target.find('.app-edit-block').length > 0;
        if (contentEditBtn && !targetEditBtn) {
            $dom.find('.app-edit-block').remove();
            $dom.find('.dm_insert_img .delete').remove();
        }

    }

    // 点击晒货
    function onClickPost ($post, $target) {
        $post.children('a').addClass('show-border');

        const isDeleteBtn = $target.hasClass('delete-post');
        const isEditBtn = $target.hasClass('edit');
        const hasEditBtn = $post.find('.app-edit-block').length > 0;
        const postId = Number($post.data('id'));

        const canSeeDirection = elementIsInWindow($post);
        const changeDirection = ($post.find('.app-edit-post-btn-wrap-bottom').length && canSeeDirection !== 'bottom') || (!$post.find('.app-edit-post-btn-wrap-bottom').length && canSeeDirection !== 'top')
        const editBlockBottomClass = canSeeDirection === 'bottom' ? 'app-edit-post-btn-wrap-bottom' : '';

        if (!hasEditBtn || changeDirection) {
            if (changeDirection) {
                $post.find('.app-edit-block').remove();
            }
            const postEditBtn = '\
                <div class="app-edit-post-btn-wrap app-edit-block ' + editBlockBottomClass + '">\
                    <span class="edit">编辑</span>\
                    <span class="delete-post"></span>\
                </div>\
            ';
            $post.append($(postEditBtn));
        } else if (isDeleteBtn) {
            removeNode($post);
        } else if (isEditBtn) {
            const postBlockId = $post.data('dom-id');
            const description = $post.find('.content').text();
            //用户选择了的图片
            const imgs = Array.prototype.map.call($post.find('.pro_img_block img'), function (ele) {
                const imgUrl =  $(ele).prop('src');
                return {
                    url:imgUrl
                };
            });
            if (window.dmEditor.editorClickEditPost) {
                let params = {
                    postBlockId: postBlockId,
                    postId: postId,
                    description: description,
                    imgs: imgs
                };
                params = JSON.stringify(params);
                window.dmEditor.editorClickEditPost(params);
            }
        }
    }

    // 点击单品
    function onClickSp ($sp, $target) {
        const isDmSp = $sp.data('type') === 'single-product';
        const isDeleteBtn = $target.hasClass('delete-sp');
        const isEditBtn = $target.hasClass('edit');
        const hasEditBtn = $sp.find('.app-edit-block').length > 0;

        const canSeeDirection = elementIsInWindow($sp);
        const changeDirection = ($sp.find('.app-edit-sp-btn-wrap-bottom').length && canSeeDirection !== 'bottom') || (!$sp.find('.app-edit-sp-btn-wrap-bottom').length && canSeeDirection !== 'top')
        const editBlockBottomClass = canSeeDirection === 'bottom' ? 'app-edit-sp-btn-wrap-bottom' : '';

        $sp.find('.sp-mask').addClass('show-border');

        if (!hasEditBtn || changeDirection) {
            if (changeDirection) {
                $sp.find('.app-edit-block').remove();
            }
            const editBtn = isDmSp ? '' : '<span class="edit">编辑</span>';
            const spEditBtn = '\
                <div class="app-edit-sp-btn-wrap app-edit-block ' + editBlockBottomClass + '">\
                    ' + editBtn + '\
                    <span class="delete-sp"></span>\
                </div>\
            ';
            $sp.append($(spEditBtn));
        } else if (isDeleteBtn) {
            removeNode($sp);
        } else if (isEditBtn) {
            const spId = $sp.data('sp-id');
            const discountId = $sp.data('sp-discount-id');
            const spBlockId = $sp.data('dom-id');
            if (window.dmEditor.editorClickEditSp) {
                let params = {
                    spId: spId,
                    discountId: discountId,
                    spBlockId: spBlockId
                };
                params = JSON.stringify(params);
                window.dmEditor.editorClickEditSp(params);
            }
        }
    }
    // 点击链接
    function onClickLink ($link, $target) {
        const isEditBtn = $target.hasClass('edit');
        const hasEditBtn = $link.find('.app-edit-block').length > 0;

        const canSeeDirection = elementIsInWindow($link);
        const changeDirection = ($link.find('.app-edit-link-btn-wrap-bottom').length && canSeeDirection !== 'bottom') || (!$link.find('.app-edit-link-btn-wrap-bottom').length && canSeeDirection !== 'top')
        const editBlockBottomClass = canSeeDirection === 'bottom' ? 'app-edit-link-btn-wrap-bottom' : '';

        if (!hasEditBtn || changeDirection) {
            if (changeDirection) {
                $link.find('.app-edit-block').remove();
            }
            const linkEditBtn = '\
                <div contenteditable="false" class="app-edit-link-btn-wrap app-edit-block ' + editBlockBottomClass + '">\
                    <span class="edit app-edit-block">编辑</span>\
                </div>\
            ';
            $link.append($(linkEditBtn));
        } else if (isEditBtn) {
            const title = $link.prop("childNodes")[0].nodeValue;
            const href = $link.attr('href');
            const linkId = $link.attr('id');
            if (window.dmEditor.editorClickEditLink) {
                let params = {
                    title: title,
                    href: href,
                    linkId: linkId
                };
                params = JSON.stringify(params);
                window.dmEditor.editorClickEditLink(params);
            }
        }
    }
    // 判断undo, redo是否可执行
    function editorHasRedoUndo() {
        let hasRedo = false, hasUndo = false, params = {};
        hasRedo = tinyMCEEditor.undoManager.hasRedo();
        hasUndo = tinyMCEEditor.undoManager.hasUndo();
        params = {
            hasRedo: hasRedo,
            hasUndo: hasUndo
        };
        return params;
    }
    
    //绑定事件
    const bindEvent = setTimeout(function () {

        const $dom = $(tinyMCEEditor.getBody());

        let hasRedoUndo = {
            hasRedo: false,
            hasUndo: false
        };
        // 内容改变时判断undo, redo是否可执行
        tinyMCEEditor.on('change undo redo', function () {
            let newHasRedoUndo = editorHasRedoUndo();
            if (hasRedoUndo.hasRedo !== newHasRedoUndo.hasRedo || hasRedoUndo.hasUndo !== newHasRedoUndo.hasUndo) {
                hasRedoUndo = newHasRedoUndo;
                if (window.dmEditor.messageAppHasRedoUndo) {
                    hasRedoUndo = JSON.stringify(hasRedoUndo);
                    window.dmEditor.messageAppHasRedoUndo(hasRedoUndo);
                }
            }
        });

        // 监听@符号，app弹起好友选择框
        tinyMCEEditor.on('input', function (e) {
            bookmarkInWindow();

            // ios键盘输入默认字体时，提示文字不会消失
            const mcePlaceholder = $dom.data('mce-placeholder');
            const content = tinyMCEEditor.getContent();
            if (mcePlaceholder) {
                $dom.removeAttr('data-mce-placeholder');
                $dom.removeAttr('aria-placeholder');
            }
            if (!content) {
                $dom.data('data-mce-placeholder', '请输入正文');
                $dom.attr('aria-placeholder', '请输入正文');
            }

            window.dmEditor.editorChangeContent();

            // 空行调整间距
            window.dmEditor.emptyPTagMargin()

        });

        //删除编辑器有时候多余出来的空标签
        // tinyMCEEditor.on('keydown',function(e){
        //     if ($dom.find("#_mce_caret").length !== 0) {
        //         $dom.find("#_mce_caret").remove();
        //     }
        // });

        tinyMCEEditor.on('nodeChange', function (e) {
            // 使用引用文字，默认提示文字不消失
            const hasBlockquote = $dom.find('blockquote, .dm-tag').length > 0;
            if (hasBlockquote) {
                $dom.removeAttr('data-mce-placeholder');
                $dom.removeAttr('aria-placeholder');
            }

            window.dmEditor.editorChangeContent();
        });

        tinyMCEEditor.on('focus', function () {
            if (window.dmEditor.appGetFocusType) {
                let params = {
                    focusType: 'content'
                }
                params = JSON.stringify(params);
                window.dmEditor.appGetFocusType(params);
            }
        });
        $dom.on('tap', function (e) {
            // 获取当前选中状态，调用APP方法
            if (window.dmEditor.editorChangeFocus) {
                window.dmEditor.editorChangeFocus();
            }
            let editableObj = { editable: true };
            let editableObjStr = '';
            const $target = $(e.target);

            const isBody = $target.hasClass('mce-content-body');
            const $mceCaretPoint = $dom.find('.mce-caret-point');
            const $post = $target.closest('.user_share_goods');
            const $img =  $target.closest('.dm_insert_img');
            const $sp = $target.closest('.dm_single_product');
            const $link = $target.closest('.dm_link');
            const $tag = $target.closest('.dm-tag');
            $dom.find('.show-outline').removeClass('show-outline');
            $dom.find('.show-border').removeClass('show-border');
                
            if ($post.length > 0) {
                removeEditBtn($post);
                onClickPost($post, $target);
                editableObj.editable = false;
            } else if ($img.length > 0) {
                removeEditBtn($img);
                onClickImage($img, $target);
                editableObj.editable = false;
            } else if ($sp.length > 0) {
                removeEditBtn($sp);
                onClickSp($sp, $target);
                editableObj.editable = false;
            } else if ($link.length > 0) {
                removeEditBtn($link);
                onClickLink($link, $target);
                editableObj.editable = false;
            } else if ($tag.length > 0) {
                editableObj.editable = false;
                $dom.find('.app-edit-block').remove();
                $dom.find('.dm_insert_img .delete').remove();
            } else {
                editableObj.editable = true;
                $dom.find('.app-edit-block').remove();
                $dom.find('.dm_insert_img .delete').remove();
            }
            // 安卓弹起键盘
            if ($mceCaretPoint.length && isBody) {
                if (window.dmEditor.showKeyboard) {
                    window.dmEditor.showKeyboard();
                }
            } else if (!editableObj.editable) {
                $('.title-tip').hide();
                if (window.dmEditor.hideKeyboard) {
                    window.dmEditor.hideKeyboard();
                }
            }

            if (window.dmEditor.messageAppEditable) {
                editableObjStr = JSON.stringify(editableObj);
                window.dmEditor.messageAppEditable(editableObjStr);
            }
        }); 

        function prevCharIsSpace() {
            // debugger;
            var start = tinyMCEEditor.selection.getRng(true).startOffset,
              text = tinyMCEEditor.selection.getRng(true).startContainer.data || "",
              charachter = text.substr(start > 0 ? start - 1 : 0, 1);

            return !!$.trim(charachter).length ? false : true;
        }

        tinyMCEEditor.on("keydown", function (e) {
            const $curNode = $(tinyMCEEditor.selection.getNode());
            
            if (e.keyCode == 13) {
                const focusAttributes = JSON.parse(window.dmEditor.editorGetFocusAttributes());
                const isBlockquote = $curNode.closest('.dm_blockquote').length > 0;
                const isBold = focusAttributes.includes('bold');
                const isRedColor = focusAttributes.includes('textColorRed');

                if (isBlockquote) {
                    const removeBlockquote = setTimeout(function () {
                        window.dmEditor.toggleBlockquoteFormat();
                        clearTimeout(removeBlockquote);
                    }, 100);
                }
                if (isBold) {
                    window.dmEditor.editorOperation(JSON.stringify({styleName: "bold"}));
                }

                if (isRedColor) {
                    window.dmEditor.editorOperation(JSON.stringify({styleName: "textColorRed", styleContent: 0}));
                }

                // 空行调整间距
                window.dmEditor.emptyPTagMargin()
            }
            if (e.keyCode === 8 || e.keyCode == 13) {
                if (window.dmEditor.editorChangeFocus) {
                    window.dmEditor.editorChangeFocus();
                }
            }

            // if (e.keyCode === 8) {
            //     const hasText = !!$curNode.text();
            //     let preEleIsBlock = false;

            //     if (!hasText) {
            //         const reg = /(dm\_insert\_img)|(user\_share\_goods)|(dm\_single\_product)/g;
            //         const $curNodePrev =  $curNode.prev();
            //         const curNodePrevClass = $curNodePrev.attr('class');
            //         preEleIsBlock = curNodePrevClass ? curNodePrevClass.match(reg) : false;
            //         if (preEleIsBlock) {
            //             e.preventDefault();
            //             const $post = $curNodePrev.closest('.user_share_goods');
            //             const $img =  $curNodePrev.closest('.dm_insert_img');
            //             const $sp = $curNodePrev.closest('.dm_single_product');
            //             $dom.find('.show-outline').removeClass('show-outline');
            //             $dom.find('.show-border').removeClass('show-border');
                        
            //             if ($post.length > 0) {
            //                 removeEditBtn($post);
            //                 onClickPost($post, $curNodePrev);
            //             } else if ($img.length > 0) {
            //                 removeEditBtn($img);
            //                 onClickImage($img, $curNodePrev);
            //             } else if ($sp.length > 0) {
            //                 removeEditBtn($sp);
            //                 onClickSp($sp, $curNodePrev);
            //             }
            //         }
            //     }
            // }
        });
        
        //at好友事件，用于插入一个@ document node
        // tinyMCEEditor.on("keypress", function(e) {
        //     const keyCode = e.which || e.keyCode || e.data;
        //     const asciiNum = typeof data === 'string' ? data.charCodeAt() : 0;
        //     tinyMCEEditor.dmCommon.editorToast('keypress keycode = ' + keyCode);
        //     var delimiterIndex = $.inArray(
        //       String.fromCharCode(keyCode),
        //       '@'
        //     );
        //     if (keyCode === '@' || asciiNum === 65312 || delimiterIndex > -1) {
        //       e.preventDefault();
        //       var rawHtml =
        //         '<span id="delimiter">@</span>'
        //       tinyMCEEditor.execCommand("mceInsertContent", false, rawHtml);
        //       tinyMCEEditor.focus();
        //     }
        // });

        $('#tiny-mask').remove();
        
        clearTimeout(bindEvent);
    }, 1000);
});
