
tinymce.PluginManager.add('dmstyleoperation', function(tinyMCEEditor) {

    function toggleBlockquoteFormat( alignment ) {
        alignment = alignment || 'right';
        tinyMCEEditor.undoManager.add();
        if (!tinyMCEEditor.formatter.match('cc_blockquote_format')) {
            tinyMCEEditor.formatter.apply('cc_blockquote_format');
            var $selectedElement = $(tinyMCEEditor.selection.getNode());

            if ($selectedElement.is('blockquote')) {
                $blockquote = $selectedElement;
            } else {
                $blockquote = $selectedElement.closest('blockquote');
            }

            $blockquote.addClass('dm_blockquote-' + alignment);

            var $img = $blockquote.find('img');
            if ($img.length) {
                $blockquote.addClass('dm_blockquote-image');
            } else {
                $blockquote.addClass('dm_blockquote-text');
            }
        } else {
            var $selectedElement = $(tinyMCEEditor.selection.getNode());
            var $blockquote = $selectedElement.closest('.dm_blockquote');

            $blockquote.find('span.dm_blockquote-border').remove();

            $blockquote.removeClass('dm_blockquote-text dm_blockquote-image dm_blockquote-' + alignment);

            tinyMCEEditor.formatter.remove('cc_blockquote_format');
        }

        tinyMCEEditor.nodeChanged(); // refresh the button state
    }

    tinyMCEEditor.on('init', function(e) {
        tinyMCEEditor.formatter.register(
            'cc_blockquote_format', {
                block: 'blockquote',
                classes: ['dm_blockquote'],
                //attributes: {'class': 'dm_blockquote-%value'},  // workaround for http://www.tinymce.com/develop/bugtracker_view.php?id=6472
                wrapper: true
            }
        );
    });


    const attributes = [
        { name: 'fontSize', type: 'one' },
        // { name: 'foreColor', type: 'one' },
        { name: 'bold', type: 'two' },
        { name: 'color', type: 'two' },
        { name: 'italic', type: 'two' },
        { name: 'insertOrderedList', type: 'InsertOrderedList' },
        { name: 'insertUnorderedList', type: 'InsertUnorderedList' },
        { name: 'justifyRight', type: 'two' },
        { name: 'justifyLeft', type: 'two' },
        { name: 'justifyFull', type: 'two' },
        { name: 'justifyCenter', type: 'two' },
        { name: 'h2', type: 'header' },
        { name: 'h3', type: 'header' },
        { name: 'undo', type: 'undoManager' },
        { name: 'redo', type: 'undoManager' },
        { name: 'formatBlock', type: 'formatBlock' },
        { name: 'textColorRed', type: 'textColorRed' },
    ];
    
    const headFontSize = {
        h1: '20px',
        h2: '16px',
    };

    const blockMap = {
        ol:'insertOrderedList',
        ul:'insertUnorderedList',
        h2:'H2',
        h3:'H3'
    }
    
     //获取选中文字及range
     function getSelectionText() {
        const selection = tinyMCEEditor.selection;
        const bookmark = selection.getBookmark();
        const selectionText = selection.getContent({format: 'text'}) || '';
        const startOffset = selection.getRng().startOffset;
        const endOffset = selection.getRng().endOffset;
        const hasSelectText = endOffset !== startOffset;
        let result = {
            selectionText: selectionText,
            startOffset: hasSelectText ? startOffset : 0,
            endOffset: hasSelectText ? endOffset : 0,
            bookmark: bookmark
        };
        result = JSON.stringify(result);
        return result;
    }

    // 获取光标所在位置的元素属性
    // 返回的结果有[h1,h2,bold,italic,insertOrderedList,insertUnorderedList,justifyRight,justifyLeft,justifyFull,justifyCenter]
    function editorGetFocusAttributes () {
        let tagName = '', focusAttributes = [], isH2 = false, isH3 = false, isTextColorRed = false;
        const node = tinyMCEEditor.selection.getNode();
        // tagName = tinyMCE.editors.mytextarea.selection.getNode().tagName;
        // parentTag = 
        // isH2 = tagName.match('H2');
        // isH3 = tagName.match('H3');
        const $ele = $(node);
        isTextColorRed = $ele.css('color') === ('rgb(237, 25, 100)' || '#ed1964');
        if (node.tagName == 'H2' || $ele.closest('.dm_h2').length || $ele.closest('h2').length) {
            isH2 = true;
        }
        if (node.tagName == 'H3' || $ele.closest('.dm_h3').length || $ele.closest('h3').length) {
            isH3 = true;
        }
        attributes.forEach(i => {
            const attributeName = i.name;
            const hasAttribute = tinyMCEEditor.queryCommandValue(attributeName);
            if (attributeName === 'formatBlock') {
                const isBlockquote = $(tinyMCEEditor.selection.getNode()).closest('blockquote').length > 0;
                if (isBlockquote) {
                    focusAttributes = [ ...focusAttributes, 'formatBlock' ];
                }
            } else if (hasAttribute === 'true') {
                focusAttributes = [ ...focusAttributes, attributeName ];
            }
        });

        if (isH2) {
            focusAttributes = [ ...focusAttributes, 'h2' ];
            // focusAttributes = focusAttributes.filter(i => i !== 'bold');
        } else if (isH3) {
            focusAttributes = [ ...focusAttributes, 'h3' ];
            // focusAttributes = focusAttributes.filter(i => i !== 'bold');
        }
        if (isTextColorRed) {
            focusAttributes = [ ...focusAttributes, 'textColorRed' ];
        }
        focusAttributes = JSON.stringify(focusAttributes);
        // tinyMCEEditor.dmCommon.editorToast(focusAttributes);
        return focusAttributes;
    }

    // 修改样式，初始化列表
    function editorOperation (data) {
        data = JSON.parse(data);
        const styleName = data.styleName;
        const styleContent = Number(data.styleContent);
        // const node = tinyMCEEditor.selection.getNode();
        const attribute = attributes.find(i => i.name === styleName);
        if (attribute) {
            const type = attribute.type;
            if (type === 'header') {//标题h1 h2
                // 值为1时添加标题，为0时取消标题
                if (styleContent) {
                    beforeSetTitleBlock();
                    tinyMCEEditor.formatter.apply(styleName);
                } else {
                    tinyMCEEditor.formatter.remove(styleName);
                    // tinyMCEEditor.formatter.apply('p');
                }
                tinyMCEEditor.undoManager.add();
            } else if (type === 'two') {//加粗斜体
                tinyMCEEditor.execCommand(styleName);
                tinyMCEEditor.undoManager.add();
            } else if (type === 'textColorRed') {
                if (styleContent) {
                    tinyMCEEditor.formatter.apply('forecolor');
                } else {
                    tinyMCEEditor.formatter.remove('forecolor');
                }
                tinyMCEEditor.undoManager.add();
            } else  if (type === 'undoManager') {//撤销重做
                if (styleName === 'undo') {
                    tinyMCEEditor.undoManager.undo();
                } else if (styleName === 'redo') {
                    tinyMCEEditor.undoManager.redo();
                }
            } else if (type === 'formatBlock') {//引用样式
                beforeSetBlockquote();
                toggleBlockquoteFormat();
            } else if (type === 'InsertUnorderedList' || type === 'InsertOrderedList') {//有序无序
                if (styleContent) {
                    if (getListType()) {
                        tinyMCEEditor.execCommand('RemoveList');
                    }
                    beforeSetListBlock();
                    tinyMCEEditor.execCommand(type);
                }else{
                    tinyMCEEditor.execCommand('RemoveList');
                }
            }
        }
    }

    function removeBold(){
        const $dom = $(tinyMCEEditor.getBody());
        const $curNode = $(tinyMCEEditor.selection.getNode());

        const $curNodeParent = $curNode.parents('strong');
        const $node = $curNodeParent.length ? $curNodeParent : $curNode;
        if ($node[0].nodeName == 'STRONG') {
            const selection = $node[0];
            const selectionText = $node.text();
            if ($dom.find("#_mce_caret").length !== 0) {
                $dom.find("#_mce_caret").remove();
            }
            tinyMCEEditor.dom.remove(selection);
            tinyMCEEditor.execCommand('mceInsertContent', false, selectionText);
        }
    }

    function isBlockquote(){
        const node = tinyMCEEditor.selection.getNode();
        const $node = $(node);
        if (node.nodeName == 'BLOCKQUOTE' || $node.closest('.dm_blockquote').length) {
            return true;
        }else{
            return false;
        }
    }

    function getListType(){
        const node = tinyMCEEditor.selection.getNode();
        const $node = $(node);
        const isUlOl = node.nodeName == 'LI' || $node.closest('li').length || node.nodeName == 'UL' || node.nodeName == 'OL';
        if (isUlOl) {
            if ($node.closest('.dm_ol').length || $node.closest('ol').length) {
                return 'ol';
            }else if($node.closest('.dm_ul').length || $node.closest('ul').length){
                return 'ul';
            }
        }else{
            return false;
        }
    }

    function getTitleType(){
        const node = tinyMCEEditor.selection.getNode();
        const $node = $(node);
        if (node.nodeName == 'H2' || $node.closest('.dm_h2').length) {
            return 'h2';
        }else if(node.nodeName == 'H3' || $node.closest('.dm_h3').length){
            return 'h3';
        }else{
            return ''
        }
    }

    function beforeSetTitleBlock(){
        removeBold();
        const node = tinyMCEEditor.selection.getNode();
        $(node).removeClass('dm_h3 dm_h2 app-empty-p');
        const listType = getListType(node);
        if (isBlockquote(node)) {
            toggleBlockquoteFormat();
        }
        if (listType) {
            tinyMCEEditor.execCommand(blockMap[listType]);
        }
    }

    function beforeSetBlockquote(){
        removeBold();
        const node = tinyMCEEditor.selection.getNode();
        const titleType = getTitleType(node)
        const listType = getListType(node)
        if (titleType) {
            tinyMCEEditor.formatter.remove(titleType);
        }
        if (listType) {
            tinyMCEEditor.execCommand(blockMap[listType]);
        }
    }

    function beforeSetListBlock(){
        removeBold();
        const node = tinyMCEEditor.selection.getNode();
        const titleType = getTitleType(node);
        if (isBlockquote(node)) {
            toggleBlockquoteFormat();
        }
        if (titleType) {
            tinyMCEEditor.formatter.remove(titleType);
        }
    }

    // 无序列表、有序列表、引用内插入块元素需再起一行插入块元素
    function cancleListBlockquote () {
        const node = tinyMCEEditor.selection.getNode();
        const listType = getListType(node);
        if (isBlockquote(node)) {
            toggleBlockquoteFormat();
        }
        if (listType) {
            tinyMCEEditor.execCommand(blockMap[listType]);
        }
    }

    //修改图片链接
    function editorChangeImageUrl (data) {
        data = JSON.parse(data);
        const $dom = $(tinyMCEEditor.getBody());
        const imgId = data.imgId;
        const imageUrl = data.imageUrl;
        const $img = $($dom).find('#' + imgId + ' img');
        const $gifIcon = $img.siblings('.gif-icon');
        const $newImg = '<img src="' + imageUrl + '" data-w='+data.width+' data-h="'+data.height+'" />';
        $img.replaceWith($newImg);
        if ($gifIcon.length) {
            $gifIcon.removeClass('gif-show');
        }
        tinyMCEEditor.undoManager.add();
    }

    //修改图片注释
    function editorChangeImageNode (data) {
        data = decodeURIComponent(data);
        data = JSON.parse(data);
        const $dom = $(tinyMCEEditor.getBody());
        const imgId = data.imgId;
        const node = data.node || '';
        const $imgComment = $($dom).find('#' + imgId + ' .dm_comment');
        $imgComment.text(node);
        if (node) {
            $imgComment.css('display', 'block').attr('data-mce-style', 'display: block');
        } else {
            $imgComment.css('display', 'none').attr('data-mce-style', 'display: none');
        }
        tinyMCEEditor.execCommand('mceAutoResize');
    }

    //修改A链接
    function editorChangeLink (data) {
        data = decodeURIComponent(data);
        data = JSON.parse(data);
        const $dom = $(tinyMCEEditor.getBody());
        const linkId = data.linkId;
        const href = data.href;
        const title = data.title || href;
        const $link = $dom.find('#' + linkId);
        $link.text(title);
        $link.attr('data-mce-href', href).attr('href', href).attr('title', title);
    }

    //修改单品内容
    function editorChangeSp (data) {
        data = JSON.parse(data);
        const $dom = $(tinyMCEEditor.getBody());
        const spBlockId = data.spBlockId;
        const $sp =  $dom.find('[data-dom-id="' + spBlockId + '"]');
        const spIframe = $sp.find('iframe')[0];
        $sp.find('iframe').remove();
        $sp.prepend($(spIframe));
    }

    //修改晒货内容
    function editorChangePost (data) {
        let $dom, $post, postDetail, postBlockId, imgs, imgContent = '', imgBlock, description;
        $dom = $(tinyMCEEditor.getBody());
        postDetail = decodeURIComponent(data);
        postDetail = JSON.parse(postDetail);
        postBlockId = postDetail.postBlockId;
        imgs = postDetail.imgs;
        description = postDetail.description || postDetail.newDescription;
        imgs.forEach(function (item) {
            const imgUrl = item.url;
            const imgWidth = item.url;
            const imgHeight = item.height;
            const ratio = (imgHeight / imgHeight).toFixed(2);
            const img = '<img class="pro_img" src="' + imgUrl + '" data-w="' + imgWidth + '" data-h="' + imgHeight + '" data-ratio="' + ratio + '" data-mce-src="' + imgUrl + '">';
            imgContent = imgContent + img;
        });

        imgBlock = '\
            <div class="pro_img_block">\
                ' + imgContent + '\
                <div class="dm_edit_action" style="display: none;" data-mce-style="display: none;">\
                    <button class="btn del_btn" contenteditable="false">删除</button>\
                    <button class="btn edit_btn" contenteditable="false">编辑</button>\
                </div>\
            </div>\
        '
        $post = $dom.find('[data-dom-id="' + postBlockId + '"]');
        $post.find('.content').text(description);
        $post.find('.pro_img_block').replaceWith($(imgBlock));
    }

    function h5GetKeyboardOffsetTop(data) {
        const obj = JSON.parse(data);
        const titleHeight = $('.title-wrap').height();
        // 标题高度可能是两行
        const keyboardOffsetTop = obj.keyboardOffsetTop - titleHeight;
        window.dmEditor.keyboardOffsetTop = keyboardOffsetTop;
    }

    function h5GetTextModalStatus(data) {
        const obj = JSON.parse(data);
        const textModalStatus = obj.textModalStatus;
        window.dmEditor.textModalStatus = textModalStatus;
    }

    // 通知app是否可以点击下一步
    function editorChangeContent() {
        const editorHasContent = window.dmEditor.getEditorChars().length > 4;
        if (editorHasContent !== window.dmEditor.hasContent.editor) {
            const titleHasContent = window.dmEditor.hasContent.title;
            window.dmEditor.hasContent.editor = editorHasContent;
            if (titleHasContent && window.dmEditor.messageAppHasContent) {
                const hasContent = editorHasContent;
                let parmas = {
                    hasContent: hasContent
                };
                parmas = JSON.stringify(parmas);
                window.dmEditor.messageAppHasContent(parmas);
            }
        }
    }

    function ua(){
        const ua = window.navigator.userAgent.toLocaleLowerCase();
        const isIOS = /iphone|ipad|ipod/.test(ua);
        const isAndroid = /android/.test(ua);
        return deviceType = isIOS?'ios':isAndroid?'android':'';
    }

    // 手动失焦
    // function editorBlur() {
    //     if (ua() == 'ios') {
    //         $('#hidden-input').focus();
    //     }
    // }
    window.dmEditor.cancleListBlockquote = cancleListBlockquote;
    window.dmEditor.toggleBlockquoteFormat = toggleBlockquoteFormat;

    // 给APP使用的api
    window.dmEditor.editorChangeLink = editorChangeLink;
    window.dmEditor.editorChangePost = editorChangePost;
    window.dmEditor.editorChangeSp = editorChangeSp;
    window.dmEditor.editorChangeImageNode = editorChangeImageNode;
    window.dmEditor.editorChangeImageUrl = editorChangeImageUrl;
    window.dmEditor.editorGetFocusAttributes = editorGetFocusAttributes;
    window.dmEditor.editorOperation = editorOperation;
    window.dmEditor.getSelectionText = getSelectionText;
    window.dmEditor.h5GetKeyboardOffsetTop = h5GetKeyboardOffsetTop;
    window.dmEditor.h5GetTextModalStatus = h5GetTextModalStatus;
    window.dmEditor.editorChangeContent = editorChangeContent;
    // window.dmEditor.editorBlur = editorBlur;
});
 